using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Xml;

namespace H2Forge.Map.Plugins
{
    public class Reader
    {
        #region Instances
        private string __directory;
        #endregion

        #region Accessors
        /// <summary>
        /// Halo 2 Plugin Directory
        /// </summary>
        public string Directory { get { return __directory; } set { if (value.Substring(value.Length - 1, 1) != "\\") value += "\\"; __directory = value; } }
        #endregion

        public Reader(string pluginDir)
        {
            Directory = pluginDir;
        }

        public Items.Base[] ReadAllItems(string TagClass)
        {
            if (TagClass.Substring(3, 1) == " ") TagClass = TagClass.Substring(0, 3);

            XmlTextReader plugin = new XmlTextReader(Directory + TagClass + ".ent");

            Items.Base[] items = new Items.Base[0];

            int index = 0;

            while (plugin.EOF == false)
            {
                plugin.Read();
                if (plugin.Name != "revision")
                {
                    Array.Resize<Items.Base>(ref items, items.Length + 1);
                    index = items.Length - 1;
                    items[index] = new Items.Base();
                    items[index].Type = plugin.Name;
                    items[index].Name = plugin.GetAttribute("name");
                    items[index].Offset = Convert.ToInt32(plugin.GetAttribute("offset"));
                    items[index].Size = Convert.ToInt32(plugin.GetAttribute("size"));
                    items[index].Value = Convert.ToInt32(plugin.GetAttribute("value"));

                    string tempVis = plugin.GetAttribute("visible");
                    if (tempVis == "true" || tempVis == "True")
                        items[index].Visible = true;
                    else
                        items[index].Visible = false;
                }
            }

            return items;
        }

        public Items.ReflexiveTree[] ReadAllReflexives(string TagClass)
        {
            if (TagClass.Substring(3, 1) == " ") TagClass = TagClass.Substring(0, 3);

            XmlTextReader plugin = new XmlTextReader(Directory + TagClass + ".ent");
            plugin.Read();
            Items.ReflexiveTree[] items = LoadXml(new System.IO.StringReader(plugin.ReadOuterXml()));
            return items;
        }

        private Items.ReflexiveTree[] LoadXml(System.IO.StringReader xmlText)
        {
            XmlTextReader xml = new XmlTextReader(xmlText);

            Items.ReflexiveTree[] Reflex = new Items.ReflexiveTree[0];

            int index = 0;
            xml.Read();
            while (xml.EOF == false)
            {
                xml.Read();
                //MessageBox.Show(xml.Name);
                if (xml.Name == "struct" && xml.HasAttributes == true)
                {
                    Array.Resize<Items.ReflexiveTree>(ref Reflex, Reflex.Length + 1);
                    index = Reflex.Length - 1;
                    Reflex[index] = new Items.ReflexiveTree();
                    Reflex[index].Name = xml.GetAttribute("name");
                    Reflex[index].Offset = Convert.ToInt32(xml.GetAttribute("offset"));
                    Reflex[index].ChunkSize = Convert.ToInt32(xml.GetAttribute("size"));

                    Reflex[index].Reflexives = LoadXml(new System.IO.StringReader(xml.ReadOuterXml()));
                }
            }

            return Reflex;
        }
    }
}
