using System;
using System.Collections.Generic;
using System.Text;

namespace H2Forge.Map.Stream.Divisions
{
    public class Header
    {
        #region Instances
        private MapStream __mapStream;
        #endregion

        #region Accessors
        private MapStream ParentMapStream { get { return __mapStream; } }
        #endregion

        #region Data
        /// <summary>
        /// Read Or Write Head
        /// </summary>
        public string Head { get { return ParentMapStream.ReadReverseStringAt(0, 4, false, false); } set { ParentMapStream.WriteReverseStringAt(0, value, false); } }
        /// <summary>
        /// Read Or Write Version
        /// </summary>
        public int Version { get { return ParentMapStream.ReadInt32At(4, false); } set { ParentMapStream.WriteAt(4, value, false); } }
        /// <summary>
        /// Read Or Write Filesize
        /// </summary>
        public int Filesize { get { return ParentMapStream.ReadInt32At(8, false); } set { ParentMapStream.WriteAt(8, value, false); } }
        /// <summary>
        /// Read Or Write IndexOffset
        /// </summary>
        public int IndexOffset { get { return ParentMapStream.ReadInt32At(16, false); } set { ParentMapStream.WriteAt(16, value, false); ParentMapStream.IndexOffset = value; } }
        /// <summary>
        /// Read Or Write IndexSize
        /// </summary>
        public int IndexSize { get { return ParentMapStream.ReadInt32At(20, false); } set { ParentMapStream.WriteAt(20, value, false); } }
        /// <summary>
        /// Calculate MetaStart
        /// </summary>
        public int MetaStart { get { return IndexOffset + IndexSize; } }
        /// <summary>
        /// Read Or Write MetaSize
        /// </summary>
        public int MetaSize { get { return ParentMapStream.ReadInt32At(24, false); } set { ParentMapStream.WriteAt(24, value, false); } }
        /// <summary>
        /// Read Or Write NonRawSize
        /// </summary>
        public int NonRawSize { get { return ParentMapStream.ReadInt32At(28, false); } set { ParentMapStream.WriteAt(28, value, false); } }
        /// <summary>
        /// Read Or Write Origin
        /// </summary>
        public string Origin { get { return ParentMapStream.ReadStringAt(32, 256, true, false); } set { ParentMapStream.WriteAt(32, value, false); } }
        /// <summary>
        /// Read Or Write BuildDate
        /// </summary>
        public string BuildDate { get { return ParentMapStream.ReadStringAt(288, 32, true, false); } set { ParentMapStream.WriteAt(288, value, false); } }
        /// <summary>
        /// Read Or Write MapType
        /// </summary>
        public int MapTypeInt { get { return ParentMapStream.ReadInt32At(320, false); } set { ParentMapStream.WriteAt(320, value, false); } }
        /// <summary>
        /// Read Or Write Enum Value Of Map Type
        /// </summary>
        public Data.Types.MapTypes MapType { get { return (Data.Types.MapTypes)ParentMapStream.ReadInt32At(320, false); } set { ParentMapStream.WriteAt(320, (int)value, false); } }
        /// <summary>
        /// Read Or Write StrangeFileStringsOffset
        /// </summary>
        public int StrangeFileStringsOffset { get { return ParentMapStream.ReadInt32At(340, false); } set { ParentMapStream.WriteAt(340, value, false); } }
        /// <summary>
        /// Read Or Write StrangeFileStringsSize
        /// </summary>
        public int StrangeFileStringsSize { get { return ParentMapStream.ReadInt32At(344, false); } set { ParentMapStream.WriteAt(344, value, false); } }
        /// <summary>
        /// Read Or Write StringsOffset128
        /// </summary>
        public int StringsOffset128 { get { return ParentMapStream.ReadInt32At(352, false); } set { ParentMapStream.WriteAt(352, value, false); } }
        /// <summary>
        /// Read Or Write StringCount
        /// </summary>
        public int StringCount { get { return ParentMapStream.ReadInt32At(356, false); } set { ParentMapStream.WriteAt(356, value, false); } }
        /// <summary>
        /// Read Or Write StringsSize
        /// </summary>
        public int StringsSize { get { return ParentMapStream.ReadInt32At(360, false); } set { ParentMapStream.WriteAt(360, value, false); } }
        /// <summary>
        /// Read Or Write StringsIndex
        /// </summary>
        public int StringsIndex { get { return ParentMapStream.ReadInt32At(364, false); } set { ParentMapStream.WriteAt(364, value, false); } }
        /// <summary>
        /// Read Or Write StringsOffset
        /// </summary>
        public int StringsOffset { get { return ParentMapStream.ReadInt32At(368, false); } set { ParentMapStream.WriteAt(368, value, false); } }
        /// <summary>
        /// Read Or Write InternalName
        /// </summary>
        public string InternalName { get { return ParentMapStream.ReadStringAt(408, 32, true, false); } set { ParentMapStream.WriteAt(348, value, false); } }
        /// <summary>
        /// Read Or Write ScenarioName
        /// </summary>
        public string ScenarioName { get { return ParentMapStream.ReadStringAt(444, 256, true, false); } set { ParentMapStream.WriteAt(444, value, false); } }
        /// <summary>
        /// Read Or Write FileTableCount
        /// </summary>
        public int FileTableCount { get { return ParentMapStream.ReadInt32At(704, false); } set { ParentMapStream.WriteAt(704, value, false); } }
        /// <summary>
        /// Read Or Write FileTableOffset
        /// </summary>
        public int FileTableOffset { get { return ParentMapStream.ReadInt32At(708, false); } set { ParentMapStream.WriteAt(708, value, false); ParentMapStream.FileTableOffset = value; } }
        /// <summary>
        /// Read Or Write FileTableSize
        /// </summary>
        public int FileTableSize { get { return ParentMapStream.ReadInt32At(712, false); } set { ParentMapStream.WriteAt(712, value, false); } }
        /// <summary>
        /// Read Or Write FileTableIndexOffset
        /// </summary>
        public int FileTableIndexOffset { get { return ParentMapStream.ReadInt32At(716, false); } set { ParentMapStream.WriteAt(716, value, false); } }
        /// <summary>
        /// Read Or Write Checksum
        /// </summary>
        public int Checksum { get { return ParentMapStream.ReadInt32At(720, false); } set { ParentMapStream.WriteAt(720, value, false); } }
        /// <summary>
        /// Read Or Write Foot
        /// </summary>
        public string Foot { get { return ParentMapStream.ReadReverseStringAt(2044, 4, true, false); } set { ParentMapStream.WriteReverseStringAt(2044, value, false); } }
        #endregion

        #region New
        public Header(MapStream mapStream)
        {
            __mapStream = mapStream;
            mapStream.IndexOffset = IndexOffset;
            mapStream.MetaStart = MetaStart;
        }
        #endregion

        public void WriteFromDataHeader(Map.Data.Divisions.Header header)
        {
            Head = header.Head;
            Version = header.Version;
            Filesize = header.Filesize;
            IndexOffset = header.IndexOffset;
            IndexSize = header.IndexSize;
            MetaSize = header.MetaSize;
            NonRawSize = header.NonRawSize;
            Origin = header.Origin;
            BuildDate = header.BuildDate;
            MapTypeInt = header.MapTypeInt;

            StrangeFileStringsSize = header.StrangeFileStringsSize;
            StrangeFileStringsOffset = header.StrangeFileStringsOffset;
            StringsOffset128 = header.StringsOffset128;
            StringCount = header.StringCount;
            StringsSize = header.StringsSize;
            StringsIndex = header.StringsIndex;
            StringsOffset = header.StringsOffset;
            InternalName = header.InternalName;
            ScenarioName = header.ScenarioName;
            FileTableCount = header.FileTableCount;
            FileTableOffset = header.FileTableOffset;
            FileTableSize = header.FileTableSize;
            FileTableIndexOffset = header.FileTableIndexOffset;
            Checksum = header.Checksum;
            Foot = header.Foot;
        }

        public Map.Data.Divisions.Header ReadIntoDataHeader()
        {
            return new H2Forge.Map.Data.Divisions.Header(ParentMapStream);
        }
    }
}
