using System;
using System.Collections.Generic;
using System.Text;

namespace H2Forge.Map.Stream.Divisions
{
    public class Sbsp
    {
        #region Instances
        private MapStream __mapStream;
        #endregion

        #region Accessors
        private MapStream ParentMapStream { get { return __mapStream; } }
        #endregion

        #region Data
        /// <summary>
        /// Read Or Write BSP Count
        /// </summary>
        public int BSPCount { get { return ParentMapStream.ReadInt32At(ParentMapStream.Scnr.Offset + 528, false); } set { ParentMapStream.WriteAt(ParentMapStream.Scnr.Offset + 528, value, false); } }
        /// <summary>
        /// Read Or Write BSPTransReflx
        /// </summary>
        public int BSPTransReflx { get { return ParentMapStream.ReadInt32At(ParentMapStream.Scnr.Offset + 532, false) - ParentMapStream.SecondaryMagic; } set { ParentMapStream.WriteAt(ParentMapStream.Scnr.Offset + 532, value + ParentMapStream.SecondaryMagic, false); } }

        private void InitialiseSbspInfoList()
        {
            ParentMapStream.Position = ParentMapStream.Scnr.Offset + 528;

            //chunk count and translated reflexive
            int bspCount = ParentMapStream.ReadInt32();
            int bspTransReflx = ParentMapStream.ReadInt32() - ParentMapStream.SecondaryMagic;

            ParentMapStream.SbspInfoList = new SbspInfo[bspCount];
            ParentMapStream.LightMapTags = new SortedDictionary<string, LightMap>();

            //count tmapStreamough the bsp's in the Map
            for (int x = 0; x < bspCount; x++)
            {
                //each chunk is 68 bytes long
                ParentMapStream.Position = bspTransReflx + (x * 68);

                //for storing values to come
                ParentMapStream.SbspInfoList[x] = new SbspInfo(__mapStream, (int)ParentMapStream.Position);

                //fix the current bsp's meta offset and size
                ParentMapStream.SbspTags[ParentMapStream.SbspInfoList[x].TagPath].Offset = ParentMapStream.SbspInfoList[x].MetaOffset;
                ParentMapStream.SbspTags[ParentMapStream.SbspInfoList[x].TagPath].Size = ParentMapStream.SbspInfoList[x].MetaSize;

                //this would mean a broken ID
                if (ParentMapStream.SbspInfoList[x].LightMap.TagPath == null)
                    continue; //on to the next loop

                //fix the current bsp's ltmp meta offset and size
                ParentMapStream.LightMapTags[ParentMapStream.SbspInfoList[x].LightMap.TagPath].Offset = ParentMapStream.SbspInfoList[x].LightMap.Offset;
            }
        }
        /// <summary>
        /// Get SbspInfo Stream List
        /// </summary>
        public SbspInfo[] StreamList { get { return ParentMapStream.SbspInfoList; } }

        /// <summary>
        /// Get SbspInfo Data List
        /// </summary>
        public List<Map.Data.Types.SbspInfo> DataList()
        {
            List<Map.Data.Types.SbspInfo> list = new List<H2Forge.Map.Data.Types.SbspInfo>();

            for (int i = 0; i < StreamList.Length; i++)
            {
                list[i].ID = StreamList[i].ID;
                list[i].Magic = StreamList[i].Magic;
                list[i].MetaOffset = StreamList[i].MetaOffset;
                list[i].MetaSize = StreamList[i].MetaSize;
                list[i].PointerOffset = StreamList[i].PointerOffset;
                list[i].TagPath = StreamList[i].TagPath;

                list[i].LightMap = new H2Forge.Map.Data.Types.LightMap(ParentMapStream, i);
            }

            return list;
        }

        /// <summary>
        /// Get ScnrPath
        /// </summary>
        public string ScnrPath { get { return ParentMapStream.Tags.ReadTagPath(ParentMapStream.Index.ScnrID); } set { ParentMapStream.Tags.WriteTagPath(ParentMapStream.Index.ScnrID, value); } }
        #endregion

        public Sbsp(MapStream mapStream, bool Initialise)
        {
            __mapStream = mapStream;
            ParentMapStream.Scnr = mapStream.Tags.ReadTag(ParentMapStream.Index.ScnrID);
            if (Initialise) InitialiseSbspInfoList();
        }
    }
}
