using System;
using System.Collections.Generic;
using System.Text;

namespace H2Forge.Map.Stream.Divisions
{
    public class SbspInfo
    {
        #region Instances
        private MapStream __mapStream;
        private int __startOffset;
        #endregion

        #region Accessors
        public MapStream ParentMapStream { get { return __mapStream; } }
        public int StartOffset { get { return __startOffset; } set { __startOffset = value; } }
        #endregion

        #region Data
        /// <summary>
        /// ID
        /// </summary>
        public int ID { get { return ParentMapStream.ReadInt32At(StartOffset + 16, false); } set { ParentMapStream.WriteAt(StartOffset + 16, value, false); } }
        /// <summary>
        /// LightMap ID
        /// </summary>
        private int LightMapID { get { return ParentMapStream.ReadInt32At(StartOffset + 20, false); } set { ParentMapStream.WriteAt(StartOffset + 20, value, false); } }
        /// <summary>
        /// Calculate PointerOffset
        /// </summary>
        public int PointerOffset { get { return StartOffset - ParentMapStream.Scnr.Offset; } }
        /// <summary>
        /// MetaOffset
        /// </summary>
        public int MetaOffset { get { return ParentMapStream.ReadInt32At(StartOffset, false) & (int)0x3FFFFFFF; } set { ParentMapStream.WriteAt(StartOffset, value, false); } }
        /// <summary>
        /// MetaSize
        /// </summary>
        public int MetaSize { get { return ParentMapStream.ReadInt32At(StartOffset + 4, false); } set { ParentMapStream.WriteAt(StartOffset + 4, value, false); } }
        /// <summary>
        /// Magic
        /// </summary>
        public int Magic { get { return ParentMapStream.ReadInt32At(StartOffset + 8, false) - MetaOffset; } set { ParentMapStream.WriteAt(StartOffset + 8, value + MetaOffset, false); } }
        /// <summary>
        /// TagPath
        /// </summary>
        public string TagPath { get { return ParentMapStream.Tags.ReadTagPath(ID); } set { ParentMapStream.Tags.WriteTagPath(ID, value); } }
        /// <summary>
        /// LightMap
        /// </summary>
        public LightMap LightMap { get { return ParentMapStream.LightMapTags[ParentMapStream.Tags.ReadTagPath(LightMapID)]; } set { ParentMapStream.LightMapTags[ParentMapStream.Tags.ReadTagPath(LightMapID)] = value; } }
        #endregion

        #region New
        public SbspInfo(MapStream mapStream, int startOffset)
        {
            __mapStream = mapStream;
            StartOffset = startOffset;
        }
        #endregion
    }
}
